'use strict';

var CodeMirror = require('codemirror');

CodeMirror.defineMode('jsx', function (config, parserConfig) {
  var jsMode = CodeMirror.getMode(config, 'javascript');
  var xmlMode =  CodeMirror.getMode(config, {name: 'xml', htmlMode: true});

  function js (stream, state) {
    if (
      stream.match(/^<([^>]+)/i, false)
      && (
        state.jsState.lastType === 'operator'
        || state.jsState.lastType === 'keyword c'
        || state.jsState.lastType === '=>'
        || /^[\[{}\(,;:]$/.test(state.jsState.lastType)
      )
    ) {
      state.token = xml;
      
      return xmlMode.token(stream, state.xmlState);
    }

    return jsMode.token(stream, state.jsState);
  }

  function xml (stream, state) {
    if (state.jsNext) {
      state.jsNext = false;
      state.token = js;

      return jsMode.token(stream, state.jsState);
    } else if (stream.match(/>$/i, false)) {
      state.jsNext = true;
    }

    return xmlMode.token(stream, state.xmlState);
  }

  return {
    startState: function () {
      var jsState = jsMode.startState();
      var xmlState = xmlMode.startState();

      return {token: js, jsState: jsState, xmlState: xmlState};
    },

    copyState: function (state) {
      return {
        token: state.token,
        jsNext: state.jsNext,
        jsState: CodeMirror.copyState(jsMode, state.jsState),
        xmlState: CodeMirror.copyState(xmlMode, state.xmlState)
      };
    },

    token: function (stream, state) {
      return state.token(stream, state);
    },

    blockCommentStart: '/*',
    blockCommentEnd: '*/',
    lineComment: '//',
    fold: 'brace',
    closeBrackets: '()[]{}\'\'""``',
    electricChars: '/{}:'
  };
}, 'javascript', 'xml');
